#!/usr/bin/python2
"""
 Gather and upload Insights data for
 Red Hat Insights
"""
from __future__ import print_function
import os
import sys
import six
import subprocess
from subprocess import Popen, PIPE
from distutils.version import LooseVersion
import shlex
import logging
import logging.handlers

GPG_KEY = "/etc/insights-client/redhattools.pub.gpg"

BYPASS_GPG = os.environ.get("BYPASS_GPG", "").lower() == "true"
ENV_EGG = os.environ.get("EGG")
NEW_EGG = "/var/lib/insights/newest.egg"
STABLE_EGG = "/var/lib/insights/last_stable.egg"
RPM_EGG = "/etc/insights-client/rpm.egg"
MOTD_FILE = "/etc/motd.d/insights-client"

logger = logging.getLogger(__name__)


def log(msg):
    print(msg, file=sys.stderr)


def egg_version(egg):
    '''
    Determine the egg version
    '''
    if not sys.executable:
        return None
    try:
        proc = Popen([sys.executable, '-c', 'from insights.client import InsightsClient; print(InsightsClient(None, False).version())'],
                     env={'PYTHONPATH': egg, 'PATH': os.getenv('PATH')}, stdout=PIPE, stderr=PIPE)
    except OSError:
        return None
    stdout, stderr = proc.communicate()
    if six.PY3:
        return stdout.decode('utf-8')
    else:
        return stdout


def sorted_eggs(eggs):
    '''
    Sort eggs to go into sys.path by highest version
    '''
    if len(eggs) < 2:
        # nothing to sort
        return eggs
    # default versions to 0 so LooseVersion doesn't throw a fit
    egg0_version = egg_version(eggs[0]) or '0'
    egg1_version = egg_version(eggs[1]) or '0'

    if LooseVersion(egg0_version) > LooseVersion(egg1_version):
        return eggs
    else:
        return [eggs[1], eggs[0]]


def gpg_validate(path):
    # ENV egg might be None, so check if path defined, then check if it exists
    if not (path and os.path.exists(path)):
        return False

    if BYPASS_GPG:
        return True

    if not os.path.exists(path + ".asc"):
        return False

    gpg_template = '/usr/bin/gpg --verify --keyring %s %s %s'
    cmd = gpg_template % (GPG_KEY, path + '.asc', path)
    proc = subprocess.Popen(shlex.split(cmd), stdout=PIPE, stderr=PIPE)
    proc.communicate()
    return proc.returncode == 0


def run_phase(phase, client, validated_eggs):
    """
    Call the run script for the given phase.  If the phase succeeds returns the
    index of the egg that succeeded to be used in the next phase.
    """
    insights_command = [sys.executable, os.path.join(os.path.dirname(__file__), "run.py")] + sys.argv[1:]
    config = client.get_conf()
    debug = config["debug"]

    all_eggs = [ENV_EGG, NEW_EGG] + validated_eggs

    for i, egg in enumerate(all_eggs):
        if egg is None or not os.path.isfile(egg):
            if debug:
                log("Egg does not exist: %s" % egg)
            continue
        if config['gpg'] and not client.verify(egg)['gpg']:
            log("WARNING: GPG verification failed.  Not loading egg: %s" % egg)
            continue
        if debug:
            log("Attempting %s with egg: %s" % (phase['name'], egg))

        # setup the env
        insights_env = {
            "INSIGHTS_PHASE": str(phase['name']),
            "PYTHONPATH": str(egg)
        }
        env = os.environ
        env.update(insights_env)

        process = subprocess.Popen(insights_command,
                                   env=env)
        stdout, stderr = process.communicate()
        if process.returncode == 0:
            # phase successful, don't try another egg
            # update_motd_message()
            return
        if process.returncode == 1:
            # egg hit an error, try the next
            logger.debug('Attempt failed.')
        if process.returncode >= 100:
            # 100 and 101 are unrecoverable, like post-unregistration, or
            #   a machine not being registered yet, or simply a 'dump & die'
            #   CLI option
            sys.exit(process.returncode % 100)
    # All attemps to run phase have failed
    sys.exit(1)


def update_motd_message():
    """
    motd displays a message about system not being registered. Once we have retrieved
    any new egg, that means we've been used at least once. We make that message
    go away by pointing /etc/motd.d/insights-client at an empty file
    It is intentional that message does not reappear if a system is then unregistered.
    """
    try:
        if os.path.isfile(NEW_EGG):
            os.symlink(os.devnull, MOTD_FILE + ".tmp")
            os.rename(MOTD_FILE + ".tmp", MOTD_FILE)
    except OSError:
        pass  # In the case of multiple processes


def _main():
    """
    attempt to update with current, fallback to rpm
    attempt to collect and upload with new, then current, then rpm
    if an egg fails a phase never try it again
    """
    if os.getuid() != 0:
        sys.exit('Insights client must be run as root.')

    # sort rpm and stable eggs after verification
    validated_eggs = sorted_eggs(
        list(filter(gpg_validate, [STABLE_EGG, RPM_EGG])))
    # if ENV_EGG was specified and it's valid, add that to front of sys.path
    #  so it can be loaded initially. keep it in its own var so we don't
    #  pass it to run_phase where we load it again
    if gpg_validate(ENV_EGG):
        valid_env_egg = [ENV_EGG]
    else:
        valid_env_egg = []

    if not validated_eggs and not valid_env_egg:
        sys.exit("No GPG-verified eggs can be found")

    # ENV egg comes first
    sys.path = valid_env_egg + validated_eggs + sys.path

    try:
        # flake8 complains because these imports aren't at the top
        import insights
        from insights.client import InsightsClient
        from insights.client.phase.v1 import get_phases

        # handle client instantation here so that it isn't done multiple times in __init__
        client = InsightsClient(True, False)  # read config, but dont setup logging
        config = client.get_conf()

        # handle log rotation here instead of core
        if os.path.isfile(config['logging_file']):
            log_handler = logging.handlers.RotatingFileHandler(
                config['logging_file'], backupCount=3)
            log_handler.doRollover()
        # we now have access to the clients logging mechanism instead of using print
        client.set_up_logging()
        logging.root.debug("Loaded initial egg: %s", os.path.dirname(insights.__file__))

        if config["version"]:
            from insights_client.constants import InsightsConstants as constants
            print("Client: %s" % constants.version)
            print("Core: %s" % client.version())
            return

        for p in get_phases():
            run_phase(p, client, validated_eggs)
    except KeyboardInterrupt:
        sys.exit('Aborting.')


if __name__ == '__main__':
    _main()



#!/usr/bin/python2


def major_version():
    with open("/etc/redhat-release") as fp:
        r = fp.read().strip()

    release = r.split("release")[1].strip()
    return release.split()[0].split(".")[0]


if __name__ == "__main__":
    print(major_version())



# !/usr/bin/python

from setuptools import setup, find_packages
import subprocess
from insights_client.major_version import major_version

rhel_version = int(major_version())


def get_version():
    f = open('insights_client/constants.py')
    for line in f:
        if 'version' in line:
            return eval(line.split('=')[-1])


VERSION = get_version().split('-')[0]
MAN_PAGE = "docs/insights-client.8"
CONF_PAGE = "docs/insights-client.conf.5"
SHORT_DESC = "Red Hat Insights"
LONG_DESC = """
Uploads insightful information to Red Hat
"""

requires = ['requests', 'PyYaml', 'six']
develop = ['flake8', 'pytest']

if __name__ == "__main__":

    subprocess.call("cat {0} | gzip > {0}.gz".format(MAN_PAGE), shell=True)
    subprocess.call("cat {0} | gzip > {0}.gz".format(CONF_PAGE), shell=True)

    # where stuff lands
    logpath = "/var/log/insights-client"
    libpath = "/var/lib/insights"
    confpath = "/etc/insights-client"
    systemdpath = "/usr/lib/systemd/system"
    man5path = "/usr/share/man/man5/"
    man8path = "/usr/share/man/man8/"
    sysconfigpath = "/etc/sysconfig/"
    conf_files = ['etc/insights-client.conf',
                  'etc/insights-client.motd',
                  'etc/.fallback.json',
                  'etc/.fallback.json.asc',
                  'etc/redhattools.pub.gpg',
                  'etc/cert-api.access.redhat.com.pem',
                  'etc/.exp.sed',
                  'etc/rpm.egg',
                  'etc/rpm.egg.asc']

    if rhel_version == 6:
        conf_files.append('etc/insights-client.cron')

    data_files = [
        # config files
        (confpath, conf_files),

        # man pages
        (man5path, ['docs/insights-client.conf.5']),
        (man8path, ['docs/insights-client.8']),

        (logpath, []),
        (libpath, []),
    ]

    if rhel_version >= 7:
        data_files.append(
            (systemdpath, ['etc/insights-client.service', 'etc/insights-client.timer'])
        )
    else:
        data_files.append(
            (sysconfigpath, ['etc/sysconfig/insights-client'])
        )

    setup(
        name="insights-client",
        author="Richard Brantley <rbrantle@redhat.com>, Jeremy Crafts <jcrafts@redhat.com>, Dan Varga <dvarga@redhat.com>",
        author_email="rbrantle@redhat.com, jcrafts@redhat.com",
        license="GPL",
        version=VERSION,
        packages=find_packages(),
        install_requires=requires,
        extras_require={'develop': requires + develop},
        include_package_data=True,
        entry_points={'console_scripts': [
            'redhat-access-insights = insights_client:_main',
            'insights-client = insights_client:_main',
        ]},
        data_files=data_files,
        description=SHORT_DESC,
        long_description=LONG_DESC
    )



#!/usr/bin/python2
import os
import sys

try:
    try:
        from insights.client.phase import v1 as client
    except ImportError as e:
        sys.exit(
            "Error importing insights.client for %s as %s: %s"
            % (os.environ["INSIGHTS_PHASE"], os.environ["PYTHONPATH"], e)
        )

    phase = getattr(client, os.environ["INSIGHTS_PHASE"])
    sys.exit(phase())
except KeyboardInterrupt:
    sys.exit(1)
